<?php

/**
 *      \file       /ouvrage/core/triggers/interface_99_modOuvrage_Workflow.class.php
 *      \ingroup    ouvrage
 *      \brief      Trigger file for create ouvrage data
 */


/**
 *      \class      InterfaceOuvrageWorkflow
 *      \brief      Classe des fonctions triggers des actions personnalisees du milestone
 */
class InterfaceOuvrageWorkflow extends DolibarrTriggers
{
	public $family = "ouvrage";
	public $picto = 'ouvrage@ouvrage';
	public $description = "Triggers of this module allows to create ouvrage data";
	public $version = '1.0.0';            // 'development', 'experimental', 'dolibarr' or version

	/**
	 *   Constructor
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		$this->db = $db;

		$this->name = preg_replace('/^Interface/i', '', get_class($this));
		$this->family = "ouvrage";
		$this->description = "Triggers of this module allows to create ouvrage data";
		$this->version = '1.0.0';            // 'development', 'experimental', 'dolibarr' or version
		$this->picto = 'ouvrage@ouvrage';
	}


	/**
	 * Trigger name
	 *
	 * @return        string    Name of trigger file
	 */
	public function getName()
	{
		return $this->name;
	}

	/**
	 * Trigger description
	 *
	 * @return        string    Description of trigger file
	 */
	public function getDesc()
	{
		return $this->description;
	}

	/**
	 * Trigger version
	 *
	 * @return        string    Version of trigger file
	 */
	public function getVersion()
	{
		global $langs;
		$langs->load("admin");

		if ($this->version == 'development') {
			return $langs->trans("Development");
		} elseif ($this->version == 'experimental') {
			return $langs->trans("Experimental");
		} elseif ($this->version == 'dolibarr') {
			return (int) DOL_VERSION;
		} elseif ($this->version) {
			return $this->version;
		} else {
			return $langs->trans("Unknown");
		}
	}

	/**
	 * Function called when a Dolibarrr business event is done.
	 * All functions "run_trigger" are triggered if file
	 * is inside directory core/triggers
	 *
	 * @param string    $action Event action code
	 * @param Object    $object Object
	 * @param User      $user   Object user
	 * @param Translate $langs  Object langs
	 * @param Conf      $conf   Object conf
	 *
	 * @return        int                        <0 if KO, 0 if no triggered ran, >0 if OK
	 */
	public function runTrigger($action, $object, User $user, Translate $langs, Conf $conf)
	{
		global $db;

		if (empty($conf->global->MAIN_MODULE_OUVRAGE)) {
			return 0;
		}

		if (!in_array($object->element, ['propal', 'commande', 'facture', 'propaldet', 'commandedet', 'facturedet'])) {
			return 0;
		}
	//	print '<pre>'.print_r($object->element,1).'</pre>';
		//exit();


		$update_actions = array('LINEPROPAL_UPDATE', 'LINEORDER_UPDATE', 'LINEBILL_UPDATE');
		$delete_actions = array('LINEPROPAL_DELETE', 'LINEORDER_DELETE', 'LINEBILL_DELETE');
		$ouvrage_actions = array('OUVRAGE_UPDATE', 'OUVRAGE_DELETE');
		$authorizedActions = array_merge($update_actions, $delete_actions);
		$authorizedActions = array_merge($authorizedActions, $ouvrage_actions);

		if (!in_array($action, $authorizedActions)) {
			return 0;
		}

		// Update Line
		if (in_array($action,
				$update_actions) && (!empty($object->fk_parent_line) || !empty($object->oldline->fk_parent_line))) {
			dol_syslog("Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->rowid);

			switch ($action) {
				case 'LINEPROPAL_UPDATE':
				case 'LINEPROPAL_MODIFY':
					include_once DOL_DOCUMENT_ROOT . '/comm/propal/class/propal.class.php';

					$workLine = new PropaleLigne($db);
					if (empty($object->fk_parent_line)) { //Compatibility V7
						$object->fk_parent_line = $object->oldline->fk_parent_line;
						$object->label = $object->oldline->label;
					}
					$workLine->fetch($object->fk_parent_line);

					$element = new Propal($db);
					$element->fetch($object->fk_propal);

					$fk_element = 'fk_propal';

					$res = $this->updateWorkChildLine($workLine, $object, $element, $fk_element);
					break;
				case 'LINEORDER_UPDATE':
				case 'LINEORDER_MODIFY':
					include_once DOL_DOCUMENT_ROOT . '/commande/class/commande.class.php';

					$workLine = new OrderLine($db);
					if (empty($object->fk_parent_line)) { //Compatibility V7
						$object->fk_parent_line = $object->oldline->fk_parent_line;
					}
					$workLine->fetch($object->fk_parent_line);

					$element = new Commande($db);
					$element->fetch($object->fk_commande);

					$fk_element = 'fk_commande';

					$res = $this->updateWorkChildLine($workLine, $object, $element, $fk_element);
					break;
				case 'LINEBILL_UPDATE' :
				case 'LINEBILL_MODIFY' :
					include_once DOL_DOCUMENT_ROOT . '/compta/facture/class/facture.class.php';

					$workLine = new FactureLigne($db);
					if (empty($object->fk_parent_line)) { //Compatibility V7
						$object->fk_parent_line = $object->oldline->fk_parent_line;
					}
					$workLine->fetch($object->fk_parent_line);

					$element = new Facture($db);
					$element->fetch($object->fk_facture);

					$fk_element = 'fk_facture';

					$res = $this->updateWorkChildLine($workLine, $object, $element, $fk_element);
					break;
			}
			return $res;
		}


		// Delete Line
		if (in_array($action,
				$delete_actions) && (!empty($object->fk_parent_line) || !empty($object->oldline->fk_parent_line))) {
			dol_syslog("Trigger '" . $this->name . "' for action '$action' launched by " . __FILE__ . ". id=" . $object->rowid);

			switch ($action) {
				case 'LINEPROPAL_DELETE':
					include_once DOL_DOCUMENT_ROOT . '/comm/propal/class/propal.class.php';

					$workLine = new PropaleLigne($db);
					if (empty($object->fk_parent_line)) { //Compatibility V7
						$object->fk_parent_line = $object->oldline->fk_parent_line;
					}
					$workLine->fetch($object->fk_parent_line);

					$element = new Propal($db);
					$element->fetch($object->fk_propal);

					$res = $this->deleteWorkChildLine($workLine, $object, $element);
					break;
				case 'LINEORDER_DELETE':
					include_once DOL_DOCUMENT_ROOT . '/commande/class/commande.class.php';

					$workLine = new OrderLine($db);
					if (empty($object->fk_parent_line)) { //Compatibility V7
						$object->fk_parent_line = $object->oldline->fk_parent_line;
					}
					$workLine->fetch($object->fk_parent_line);

					$element = new Commande($db);
					$element->fetch($object->fk_commande);

					$res = $this->deleteWorkChildLine($workLine, $object, $element);
					break;
				case 'LINEBILL_DELETE' :
					include_once DOL_DOCUMENT_ROOT . '/compta/facture/class/facture.class.php';

					$workLine = new FactureLigne($db);
					if (empty($object->fk_parent_line)) { //Compatibility V7
						$object->fk_parent_line = $object->oldline->fk_parent_line;
					}
					$workLine->fetch($object->fk_parent_line);

					$element = new Facture($db);
					$element->fetch($object->fk_facture);

					$res = $this->deleteWorkChildLine($workLine, $object, $element);
					break;
			}
		}

		if (in_array($action, $ouvrage_actions) && Ouvrage::isOuvrage($object)) {
			$element = null;

			switch ($object->element) {
				case 'propaldet' :
					include_once DOL_DOCUMENT_ROOT . '/comm/propal/class/propal.class.php';

					$element = new Propal($db);
					$element->fetch($object->fk_propal);
					break;
				case 'commandedet' :
					include_once DOL_DOCUMENT_ROOT . '/commande/class/commande.class.php';

					$element = new Commande($db);
					$element->fetch($object->fk_commande);
					break;
				case 'facturedet' :
					include_once DOL_DOCUMENT_ROOT . '/compta/facture/class/facture.class.php';

					$element = new Facture($db);
					$element->fetch($object->fk_facture);
					break;
			}

			switch ($action) {
				case 'OUVRAGE_UPDATE':
					$res = $this->updateWork($object);
					break;
				case 'OUVRAGE_DELETE':
					$res = $this->deleteWork($object, $element);
					break;
			}
		}

		return 0;
	}

	/**
	 * @param Object $workLine   Work line which is the parent of Object line
	 * @param Object $object     Object line which is actually update
	 * @param Object $element    Object containing all the lines above
	 * @param string $fk_element Label for the fk_element
	 *
	 * @return mixed
	 */
	private function updateWorkChildLine($workLine, $object, $element, $fk_element)
	{
		global $conf, $user;

		$workLine->total_ht = 0;
		$workLine->multicurrency_total_ht = 0;
		$workLine->pa_ht = 0;
		$workLine->subprice = 0;

		$totsubprice = 0;
		$element->fetch_lines();

		foreach ($element->lines as $line) {
			if ($line->fk_parent_line == $object->fk_parent_line || $line->rowid == $object->rowid) { //Compatibility V7
				$workLine->multicurrency_total_ht += $line->multicurrency_total_ht;
				$workLine->pa_ht += $line->qty * $line->pa_ht;
				$totsubprice += $line->qty * $line->subprice * ($line->situation_percent / 100);
				if ($fk_element == "fk_facture" && !empty($element->situation_counter)) {
					$workLine->subprice += $line->qty * $line->subprice;
				} else {
					$workLine->subprice += $line->qty * $line->subprice;
				}
			}
		}
		if (!empty($conf->global->OUVRAGE_USE_COST_PRICE_PER_UNIT)) {
			if ($workLine->qty != 0) {
				$workLine->subprice = $workLine->subprice / $workLine->qty;
				$workLine->pa_ht = $workLine->pa_ht / $workLine->qty;
			}
		}

		$workLine->situation_percent = price2num(($totsubprice / $workLine->subprice * $workLine->qty * 100), 'MT');
		$workLine->price = empty($workLine->qty) ? 0 : price2num(($workLine->multicurrency_total_ht / $workLine->qty), 'MT');    // InfraS change
		if ($fk_element != "fk_facture" && empty($element->situation_counter)) {
			$perc = round((100 - ($workLine->price / $workLine->subprice * 100)), 2);
		}
		$workLine->multicurrency_subprice = $workLine->subprice;
		if (empty($conf->global->OUVRAGE_USE_COST_PRICE_PER_UNIT)) {
			$workLine->remise_percent = (($perc < 100) ? $perc : 0);
		}
		$workLine->multicurrency_total_ttc = $workLine->multicurrency_total_ht * (100 + $workLine->tva_tx) / 100;
		$workLine->multicurrency_total_tva = $workLine->total_ttc - $object->multicurrency_total_ht;


		$workLine->total_ht = $workLine->multicurrency_total_ht;
		$workLine->total_tva = $workLine->multicurrency_total_tva;
		$workLine->total_ttc = $workLine->multicurrency_total_ttc;

		$ret = $workLine->update_total();
		$ret = $workLine->update($user);
		if ($object->element == 'propaldet') {
			$ret = $object->update(1);
		} else {
			$ret = $object->update($user, 1);
		}
		$element->update($user);

		return $ret;
	}

	/**
	 * @param Object $workLine Work line which is the parent of Object line
	 * @param Object $object   Object line which is actually update
	 * @param Object $element  Object containing all the lines above
	 *
	 * @return mixed
	 */
	private function deleteWorkChildLine($workLine, $object, $element)
	{
		global $user;

		$workLine->total_ht = 0;
		$workLine->multicurrency_total_ht = 0;
		$workLine->pa_ht = 0;
		$workLine->subprice = 0;
		$perc = 0;

		$element->fetch_lines();

		foreach ($element->lines as $index => $line) {
			if ($line->fk_parent_line == $object->fk_parent_line && $line->rowid != $object->rowid) {
				$workLine->multicurrency_total_ht += $line->multicurrency_total_ht;
				$workLine->pa_ht += $line->qty * $line->pa_ht;
				$workLine->subprice += $line->qty * $line->subprice;
			}
		}

		$workLine->total_ht = $workLine->multicurrency_total_ht;
		$workLine->price = empty($workLine->qty) ? 0 : price2num(($workLine->multicurrency_total_ht / $workLine->qty), 'MT');    // InfraS change

		if (!empty($workLine->subprice)) {
			$perc = round((100 - ($workLine->price / $workLine->subprice * 100)), 2);
		}

		$workLine->remise_percent = $perc < 100 ? $perc : 0;
		$workLine->multicurrency_total_ttc = price2num(($workLine->total_ht * (100 + $workLine->tva_tx) / 100), 'MT');
		$workLine->multicurrency_total_tva = price2num(($workLine->multicurrency_total_ttc - $object->multicurrency_total_ht),
			'MT');

		$workLine->total_tva = $workLine->multicurrency_total_tva;
		$workLine->total_ttc = $workLine->multicurrency_total_ttc;

		$this->updateElementTotal($element, $object);
		$this->updateElementLine($element, $object, $workLine);

		$ret = $workLine->update($user, 1);
		$ret = $element->update($user);

		return $ret;
	}

	private function updateWork($object)
	{
		global $db, $user, $conf;	// InfraS change

		$useLineVAT = isset($conf->global->OUVRAGE_USE_LINE_VAT) ? $conf->global->OUVRAGE_USE_LINE_VAT : 0;	// InfraS add
		$new_qty = GETPOST('qty', 'int');

		$rapport_qty = $object->qty / $new_qty;

		$object->tva_tx = preg_split('/\ /', GETPOST('tva_tx'))[0];
		$object->label = GETPOST('label');
		$object->qty = $new_qty;
		$object->remise_percent = GETPOST('remise_percent');
		$object->desc = dol_htmlcleanlastbr(GETPOST('product_desc', 'restricthtml'));
		$object->situation_percent = GETPOST('progress');

		if (!empty(GETPOST('buying_price'))) {
			$object->pa_ht = GETPOST('buying_price');
		}

		if (!empty(GETPOST('price_ht'))) {
			$object->subprice = GETPOST('price_ht');
		}

		if (empty($object->situation_percent)) {
			$object->total_ht = ((double) $object->qty * $object->subprice);
		} else {
			$object->total_ht = ((double) $object->qty * $object->subprice * ($object->situation_percent / 100));
		}


		$ret = $object->update_total();

		$ret = $object->element == 'propaldet' ? $object->update() : $object->update($user);    // InfraS change //TODO Change this

		if ($rapport_qty != 1 || true) {
			switch ($object->element) {
				case 'commandedet' :
					include_once DOL_DOCUMENT_ROOT . '/commande/class/commande.class.php';

					$element = new Commande($db);
					$workLine = new OrderLine($db);

					$fk_element = 'fk_commande';
					$fetch_line_function = 'fetch_lines';
					break;
				case 'facturedet' :
					include_once DOL_DOCUMENT_ROOT . '/compta/facture/class/facture.class.php';

					$element = new Facture($db);
					$workLine = new FactureLigne($db);

					$fk_element = 'fk_facture';
					$fetch_line_function = 'fetch_lines';
					break;
				case 'propaldet' :
					include_once DOL_DOCUMENT_ROOT . '/comm/propal/class/propal.class.php';

					$element = new Propal($db);
					$workLine = new PropaleLigne($db);

					$fk_element = 'fk_propal';
					$fetch_line_function = 'getLinesArray';
					break;
			}
			$idouvrage = $object->product_type == 9 ? $object->rowid : $object->fk_parent_line;

			$workLine->fetch($idouvrage);

			$workLine->total_ht = 0;
			$workLine->multicurrency_total_ht = 0;
			if (empty($conf->global->OUVRAGE_USE_COST_PRICE_PER_UNIT)) {
				$workLine->pa_ht = 0;
			}
			$workLine->subprice = 0;
			//$ouvrage->qty = ceil($ouvrage->qty / $rapport_qty);

			$element->fetch($object->{$fk_element});

			$element->total_ht = 0;
			$element->total_tva = 0;
			$element->total_ttc = 0;


			$element->{$fetch_line_function}();
			$totsubprice2 = $totsubprice3 = 0;

			foreach ($element->lines as $line) {
				if ($line->fk_parent_line == $idouvrage) {
					$line->rowid = (!empty($line->rowid) ? $line->rowid : $line->id);

					if ($object->product_type == 9) {
						$line->qty = ($line->qty / $rapport_qty);
						$line->tva_tx = $useLineVAT ? $line->tva_tx : preg_split('/\ /', GETPOST('tva_tx'))[0];	// InfraS change
						$line->remise_percent = GETPOST('remise_percent');
					}

					if ($line->remise_percent != $workLine->remise_percent) {
						$workLine->remise_percent = 0;
					}
					if ($fk_element == "fk_facture" && !empty($element->situation_counter)) {
						$line->total_ht = $line->qty * $line->subprice * ($line->situation_percent / 100);
					} else {
						$line->total_ht = $line->qty * $line->subprice;
					}

					if ($fk_element == "fk_facture" && !empty($element->situation_counter)) {
						$workLine->subprice += $line->qty * $line->subprice;
					} else {
						$workLine->subprice += $line->qty * $line->subprice;
					}

					if (empty($conf->global->OUVRAGE_USE_COST_PRICE_PER_UNIT)) {
						if ($fk_element == "fk_facture" && !empty($element->situation_counter)) {
							$workLine->pa_ht += $line->qty * $line->pa_ht;
						} else {
							$workLine->pa_ht += $line->qty * $line->pa_ht;
						}
					}

					$workLine->multicurrency_total_ht += $line->total_ht - ($line->total_ht * ($object->remise_percent / 100));
					$ret = $line->update($user);

					$line->update_total();
					if ($ret < 0) {
						$error++;
					}
					switch ($object->element) {
						case 'commandedet' :
							$element->updateline($line->rowid, $line->desc, $line->subprice, $line->qty,
								$line->remise_percent, $line->tva_tx, 0.0, 0.0, 'HT', $line->info_bits,
								$line->date_start, $line->date_end, $line->product_type, $idouvrage, 0,
								$line->fk_fournprice, $line->pa_ht, $line->label, $line->special_code, 0,
								$line->fk_unit, 0, 1);    // InfraS change $line->type to $line->product_type
							break;
						case 'facturedet' :
							if (empty($object->fk_parent_line) && !empty($element->situation_counter)) {
								$sp = $object->situation_percent;
							} else {
								$sp = $line->situation_percent;
							}

							$element->updateline($line->rowid, $line->desc, $line->subprice, $line->qty,
								$line->remise_percent, $line->date_start, $line->date_end, $line->tva_tx, 0.0, 0.0,
								'HT', $line->info_bits, $line->product_type, $idouvrage, 0, $line->fk_fournprice,
								$line->pa_ht, $line->label, $line->special_code, 0, $sp,
								$line->fk_unit, 0, 1);    // InfraS change $line->type to $line->product_type
							$totsubprice2 += $line->qty * $line->subprice * ($sp / 100);
							$totsubprice3 += $line->qty * $line->subprice;

							break;
						case 'propaldet' :
							$ret = $element->updateline($line->rowid, $line->subprice, $line->qty,
								$line->remise_percent,
								$line->tva_tx, 0.0, 0.0, $line->desc, 'HT', $line->info_bits, $line->special_code,
								$idouvrage, 0, $line->fk_fournprice, $line->pa_ht, $line->label, $line->product_type,
								$line->date_start, $line->date_end, $line->array_options, $line->fk_unit, 0,
								1);    // InfraS change $line->type to $line->product_type
							break;
					}
				}
			}

			$workLine->situation_percent = ((empty($totsubprice3)) ? 0 : price2num(($totsubprice2 / $totsubprice3 * 100),
				'MT'));
			$workLine->total_ht = $workLine->multicurrency_total_ht;
			$workLine->price = ((empty($workLine->qty)) ? 0 : price2num(($workLine->multicurrency_total_ht / $workLine->qty),
				'MT'));
			$workLine->subprice = ((empty($workLine->qty)) ? 0 : price2num(($workLine->subprice / $workLine->qty),
				'MT'));

			if ($object->product_type != 9 && $fk_element != "fk_facture" && empty($element->situation_counter)) {
				$perc = round((100 - (($workLine->total_ht / ($workLine->subprice * $workLine->qty)) * 100)), 2);
				$workLine->remise_percent = $perc < 100 ? $perc : 0;
			}

			$workLine->multicurrency_total_ttc = $workLine->total_ht * (100 + $workLine->tva_tx) / 100;
			$workLine->multicurrency_total_tva = $workLine->multicurrency_total_ttc - $object->multicurrency_total_ht;


			$workLine->total_tva = $workLine->multicurrency_total_tva;
			$workLine->total_ttc = $workLine->multicurrency_total_ttc;

			$ret = $workLine->update_total();
			$ret = $workLine->update($user);

			return $ret;
		}
	}

	private function deleteWork($workLine, $element)
	{
		global $user;

		foreach ($element->lines as $line) {
			if ($line->fk_parent_line == $workLine->id) {
				if ($element->element == 'facture') {
					$res = $line->delete();
				} else {
					$res = $line->delete($user);
				}

				if ($res < 0) {
					setEventMessage("ERROR trying to delete child line of work", 'errors');
					return -1;
				} else {
					$this->updateElementTotal($element, $line);
				}
			}
		}

		if ($res < 0) {
			setEventMessage("ERROR trying to delete line of work", 'errors');
			return -2;
		} else {
			$this->updateElementTotal($element, $workLine);
		}

		$res = $element->update($user);

		return $res;
	}

	private function updateElementTotal(&$element, &$line)
	{
		$element->total_ht -= $line->total_ht;
		$element->total_tva -= $line->total_tva;
		$element->total_ttc -= $line->total_ttc;
		$element->total -= $line->total_ttc;

		$element->multicurrency_total_ht -= $line->multicurrency_total_ht;
		$element->multicurrency_total_tva -= $line->multicurrency_total_tva;
		$element->multicurrency_total_ttc -= $line->multicurrency_total_ttc;
		$element->multicurrency_subprice -= $line->multicurrency_subprice;

		$element->total_localtax1 -= $line->total_localtax1;
		$element->total_localtax2 -= $line->total_localtax2;
	}

	private function updateElementLine(&$element, &$line, &$workline)
	{
		$i = 0;
		$found = false;

		while ($i < count($element->lines) && !$found) {
			if ($element->lines[$i]->rowid == $workline->rowid) {
				$this->updateElementTotal($element->lines[$i], $line);
				$found = true;
			}
			$i++;
		}

		$i = 0;
		$found = false;
		while ($i < count($element->lines) && !$found) {
			if ($element->lines[$i]->rowid == $line->rowid) {
				unset($element->lines[$i]);
				$found = true;
			}
			$i++;
		}
	}
}
